{
-----------------------------------------------------------------
 TpbhStatusBar version 1.0.0

 A pure VCL statusbar with additional panel features
-----------------------------------------------------------------

  Copyright (C) 2001 by Peter B. Hedlund
  All rights reserved.
  http://peterandlinda.com/components/
  support@peterandlinda.com

  All contents of this file and all other files included in the
  pbhStatusbar package are copyright (C) 2001 Peter B. Hedlund and
  shall remain the exclusive property of the author.

  Use and/or distribution of them requires acceptance of the
  following terms.

  "Author" herein refers to Peter B. Hedlund (the creator of the
  pbhStatusbar component).
  "Software" refers to all files bearing this notice, as well as
  any other files and source code included with pbhStatusbar
  (typically part of a .zip archive), and all content in them,
  regardless of whether any modifications have been made.

  1) This notice may not be removed from or altered in any source
     distribution.

  2) The Software may only be redistributed in the original
     archive form with all files included and unmodified.

  3) You are permitted to compile the Software into any kind of
     applications. ("Compile" refers to the automatic process of
     translating the Software's source code into executable Intel
     x86 machine code by the Borland Delphi or C++Builder
     compilers.)

  4) This software is provided "as-is", without any express or
     implied warranty. In no event shall the Author be held liable
     for any damages arising from the use of this software.

  5) Full backward compatibility in future versions of the Software
     is not guaranteed. In no event shall the Author be held liable
     for any inconvenience or damages arising from lack of backward
     compatibility.

  6) The Author may terminate your rights to use the Software for
     any reason at his sole discretion.

  7) If you do not agree to any of these terms, you are not
     permitted to use the Software in any way, and all copies of
     it must be deleted from your system(s).

------------------------------------------------------------------------------
 Date last modified:  November 27, 2000
------------------------------------------------------------------------------
}


unit pbhStatusBar;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, ImgList;

type

  TpbhStatusBar = class;
  TpbhStatusPanel = class;

  TpbhPanelBevelStyle = (pbsSunkenBorder, pbsNoBorder, pbsRaisedBorder, pbsFlatBorder);
  TpbhPanelDrawStyle = (pdsText, pdsOwnerDraw);

  TpbhPanelColors = class(TPersistent)
  private
    FBackColor: TColor;
    FBorderShadowColor: TColor;
    FBorderHighLightColor: TColor;
    FFlatBorderColor: TColor;
    FOwner: TpbhStatusPanel;
    procedure SetBackColor(const Value: TColor);
    procedure SetBorderHighLightColor(const Value: TColor);
    procedure SetBorderShadowColor(const Value: TColor);
    procedure SetFlatBorderColor(const Value: TColor);

  public
    constructor Create(AOwner: TpbhStatusPanel);
    procedure Assign(Source: TPersistent); override;
  published
    property BackColor: TColor read FBackColor write SetBackColor;
    property BorderHighLightColor: TColor read FBorderHighLightColor write SetBorderHighLightColor;
    property BorderShadowColor: TColor read FBorderShadowColor write SetBorderShadowColor;
    property FlatBorderColor: TColor read FFlatBorderColor write SetFlatBorderColor;
  end;

  TpbhStatusPanel = class(TCollectionItem)
  private
    FSpring: Boolean;
    FFitToContent: Boolean;
    FIdealWidth: integer;
    FMinWidth: integer;
    FImageIndex: TImageIndex;
    FText: string;
    FHint: string;
    FPanelRect: TRect;

    FTag: integer;
    FAlignment: TAlignment;

    FBevel: TpbhPanelBevelStyle;
    FVisible: Boolean;
    FColors: TpbhPanelColors;
    FOnHint: TNotifyEvent;
    FOnClick: TNotifyEvent;
    FOnDblClick: TNotifyEvent;
    FStyle: TpbhPanelDrawStyle;
    procedure SetFitToContent(const Value: Boolean);
    procedure SetHint(const Value: string);
    procedure SetIdealWidth(const Value: integer);
    procedure SetImageIndex(const Value: TImageIndex);

    procedure SetSpring(const Value: Boolean);

    procedure SetText(const Value: string);
    procedure SetTag(const Value: integer);
    procedure SetAlignment(const Value: TAlignment);
    procedure SetBevel(const Value: TpbhPanelBevelStyle);
    procedure SetVisible(const Value: Boolean);
    procedure SetColors(const Value: TpbhPanelColors);
    procedure SetStyle(const Value: TpbhPanelDrawStyle);
  protected
    function GetDisplayName: string; override;
  public

    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function GetPanelRect: TRect;
    property MinWidth: integer read FMinWidth;

  published
    property Alignment: TAlignment read FAlignment write SetAlignment;
    property Bevel: TpbhPanelBevelStyle read FBevel write SetBevel;
    property Colors: TpbhPanelColors read FColors write SetColors;
    property FitToContent: Boolean read FFitToContent write SetFitToContent;
    property Hint: string read FHint write SetHint;
    property ImageIndex: TImageIndex read FImageIndex write SetImageIndex;
    property Text: string read FText write SetText;
    property Spring: Boolean read FSpring write SetSpring;
    property Style: TpbhPanelDrawStyle read FStyle write SetStyle;
    property Tag: integer read FTag write SetTag;
    property Visible: Boolean read FVisible write SetVisible;
    property Width: integer read FIdealWidth write SetIdealWidth;

    property OnClick: TNotifyEvent read FOnClick write FOnClick;
    property OnDblClick: TNotifyEvent read FOnDblClick write FOnDblClick;
    property OnHint: TNotifyEvent read FOnHint write FOnHint;
  end;

  TpbhStatusPanels = class(TCollection)
  private
    FStatusBar: TpbhStatusBar;
    function GetItem(Index: Integer): TpbhStatusPanel;
    procedure SetItem(Index: Integer; Value: TpbhStatusPanel);
    procedure EvaluateIdealSize(Item: TCollectionItem);
    procedure ResizePanels;
  protected
    function GetOwner: TPersistent; override;
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create(pbhStatusBar: TpbhStatusBar);
    function Add: TpbhStatusPanel;
    procedure Delete(Index: integer);
    property Items[Index: Integer]: TpbhStatusPanel read GetItem write SetItem; default;
  end;

  TpbhDrawPanelEvent = procedure(StatusBar: TpbhStatusBar; Canvas: TCanvas; Panel: TpbhStatusPanel;
    const Rect: TRect) of object;

  TpbhStatusBar = class(TCustomControl)
  private
        { Private declarations }
    FPanels: TpbhStatusPanels;
    FSimplePanel: Boolean;
    FSizeGrip: Boolean;
    FUseSystemFont: Boolean;
    FSimpleText: string;
    FImages: TImageList;
    FMinHeight: integer;
    FSysFont: TFont;
    FOnHint: TNotifyEvent;
    FOnDrawPanel: TpbhDrawPanelEvent;
    procedure InitSystemFont;
    procedure SetImages(const Value: TImageList);
    procedure SetPanels(const Value: TpbhStatusPanels);
    procedure SetSimplePanel(const Value: Boolean);
    procedure SetSimpleText(const Value: string);
    procedure SetSizeGrip(const Value: Boolean);
    procedure SetUseSystemFont(const Value: Boolean);

  protected
        { Protected declarations }
    procedure Click; override;
    procedure DblClick; override;

    procedure CMFontChanged(var Msg: TMessage); message CM_FONTCHANGED;
    procedure CMColorChanged(var Msg: TMessage); message CM_COLORCHANGED;
    procedure CMHintShow(var Message: TCMHintShow); message CM_HINTSHOW;
    procedure WMNCHitTest(var Message: TWMNCHitTest); message WM_NCHITTEST;

    procedure Paint; override;
  public
        { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Canvas;
  published
        { Published declarations }
    property Align default alBottom;
    property Anchors;
        //Implement property AutoHint: Boolean;
    property Color;
    property Constraints;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Font;
    property Images: TImageList read FImages write SetImages;
    property Panels: TpbhStatusPanels read FPanels write SetPanels;

    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property SizeGrip: Boolean read FSizeGrip write SetSizeGrip;
    property SimplePanel: Boolean read FSimplePanel write SetSimplePanel;
    property SimpleText: string read FSimpleText write SetSimpleText;

    property UseSystemFont: Boolean read FUseSystemFont write SetUseSystemFont;
    property Visible;

    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDrawPanel: TpbhDrawPanelEvent read FOnDrawPanel write FOnDrawPanel;
    property OnEndDock;
    property OnEndDrag;
    property OnHint: TNotifyEvent read FOnHint write FOnHint;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
    property OnStartDock;
    property OnStartDrag;
  end;

procedure Register;

implementation

const
  m_iIconSize = 16;
  m_iDefaultHeight = 19;

procedure Register;
begin
  RegisterComponents('PBH', [TpbhStatusBar]);
end;

{ TpbhStatusPanel }

procedure TpbhStatusPanel.Assign(Source: TPersistent);
var
  SourcePanel: TpbhStatusPanel absolute Source;
begin
  if Source is TpbhStatusPanel then begin
    Alignment := SourcePanel.Alignment;
    Bevel := SourcePanel.Bevel;
    Colors := SourcePanel.Colors;
    FitToContent := SourcePanel.FitToContent;
    Hint := SourcePanel.Hint;
    ImageIndex := SourcePanel.ImageIndex;
    Text := SourcePanel.Text;
    Spring := SourcePanel.Spring;
    Style := SourcePanel.Style;
    Tag := SourcePanel.Tag;
    Visible := SourcePanel.Visible;
    Width := SourcePanel.Width;
    OnClick := SourcePanel.OnClick;
    OnDblClick := SourcePanel.OnDblClick;
    OnHint := SourcePanel.OnHint;
  end else
    inherited;

end;

constructor TpbhStatusPanel.Create(Collection: TCollection);
begin
  inherited;
  FIdealWidth := 50;
  FMinWidth := 50;
  FBevel := pbsSunkenBorder;
  FColors := TpbhPanelColors.Create(Self);

  FImageIndex := -1;
  FSpring := False;
  FFitToContent := False;
  FVisible := True;
end;

destructor TpbhStatusPanel.Destroy;
begin
  FColors.Free;
  inherited;
end;

function TpbhStatusPanel.GetDisplayName: string;
begin
  Result := Text;
  if Result = '' then Result := inherited GetDisplayName;
end;

function TpbhStatusPanel.GetPanelRect: TRect;
begin
  Result := FPanelRect;
end;

procedure TpbhStatusPanel.SetAlignment(const Value: TAlignment);
begin
  if FAlignment <> Value then begin
    FAlignment := Value;
    Changed(False);
  end;
end;

procedure TpbhStatusPanel.SetBevel(const Value: TpbhPanelBevelStyle);
begin
  if FBevel <> Value then begin
    FBevel := Value;
    Changed(False);
  end;
end;

procedure TpbhStatusPanel.SetColors(const Value: TpbhPanelColors);
begin
  FColors := Value;
end;

procedure TpbhStatusPanel.SetFitToContent(const Value: Boolean);
begin
  if FFitToContent <> Value then begin
    FFitToContent := Value;
    TpbhStatusPanels(Collection).EvaluateIdealSize(Self);
    Changed(False);
  end;
end;

procedure TpbhStatusPanel.SetHint(const Value: string);
begin
  FHint := Value;
end;

procedure TpbhStatusPanel.SetIdealWidth(const Value: integer);
begin
  if FIdealWidth <> Value then begin
    FIdealWidth := Value;
    Changed(False);
  end;
end;

procedure TpbhStatusPanel.SetImageIndex(const Value: TImageIndex);
begin
  if FImageIndex <> Value then begin
    FImageIndex := Value;
    Changed(False);
  end;
end;

procedure TpbhStatusPanel.SetSpring(const Value: Boolean);
var
  i: integer;
  p: TpbhStatusPanel;
begin
  if FSpring <> Value then begin
        //Only one panel can have this property set to True
    for i := 0 to TpbhStatusPanels(Collection).Count - 1 do begin
      p := TpbhStatusPanels(Collection).Items[i];
      if p = Self then
        FSpring := Value
      else
        p.Spring := False;
    end;
    Changed(False);
  end;
end;

procedure TpbhStatusPanel.SetStyle(const Value: TpbhPanelDrawStyle);
begin
  if FStyle <> Value then begin
    FStyle := Value;
    Changed(False);
  end;
end;

procedure TpbhStatusPanel.SetTag(const Value: integer);
begin
  FTag := Value;
end;

procedure TpbhStatusPanel.SetText(const Value: string);
begin
  if FText <> Value then begin
    FText := Value;
    Changed(False);
  end;
end;

procedure TpbhStatusPanel.SetVisible(const Value: Boolean);
begin
  if FVisible <> Value then begin
    FVisible := Value;
    Changed(False);
  end;
end;

{ TpbhStatusPanels }

function TpbhStatusPanels.Add: TpbhStatusPanel;
begin
  Result := TpbhStatusPanel(inherited Add);
end;



constructor TpbhStatusPanels.Create(pbhStatusBar: TpbhStatusBar);
begin
  inherited Create(TpbhStatusPanel);
  FStatusBar := pbhStatusBar;
end;

procedure TpbhStatusPanels.Delete(Index: integer);
begin
  inherited Delete(Index);
end;

procedure TpbhStatusPanels.EvaluateIdealSize(Item: TCollectionItem);
var
  R: TRect;
begin
  with FStatusBar do
    if HandleAllocated then begin;
      if UseSystemFont then
        Canvas.Font.Assign(FSysFont)
      else
        Canvas.Font.Assign(Font);

      with TpbhStatusPanel(Item) do begin
        DrawText(Canvas.Handle, PChar(Text), Length(Text), R, DT_CALCRECT);
        FMinWidth := R.Right - R.Left + 12;
        if ImageIndex > -1 then
          FMinWidth := FMinWidth + m_iIconSize;
      end;
    end;

end;

function TpbhStatusPanels.GetItem(Index: Integer): TpbhStatusPanel;
begin
  Result := TpbhStatusPanel(inherited GetItem(Index));
end;

function TpbhStatusPanels.GetOwner: TPersistent;
begin
  Result := FStatusBar;
end;

procedure TpbhStatusPanels.ResizePanels;
var
  tR: TRect;
  i: integer;
  iSpringIndex: integer;
  iParts: array of integer;
  iPanelCount: integer;
begin
  iPanelCount := Count;

  if (iPanelCount > 0) then begin
    iSpringIndex := -1;
    tR := FStatusBar.ClientRect;

        //Leave some space at the top
    tR.Top := tR.Top + 2;
        // Initiallly set to minimum widths:
    SetLength(iParts, iPanelCount);

        //p := Items[0];

    with Items[0] do begin
      if Visible then
        if FitToContent then
          iParts[0] := MinWidth
        else
          iParts[0] := Width
      else
        iParts[0] := 0;

            {if (ImageIndex > -1) and Visible then
                iParts[0] := iParts[0] + m_iIconSize; }

      if Spring and Visible then
        iSpringIndex := 0;
    end;

    for i := 1 to iPanelCount - 1 do
      with Items[i] do begin
        if Visible then
          if FitToContent then
            iParts[i] := iParts[i - 1] + MinWidth
          else
            iParts[i] := iParts[i - 1] + Width
        else
          iParts[i] := iParts[i - 1] + 0;

        if Spring and Visible then
          iSpringIndex := i;

                {if (ImageIndex > -1) and Visible then
                    iParts[i] := iParts[i] + m_iIconSize; }

        if (i = iPanelCount - 1) and Visible then
          iParts[i] := iParts[i] + GetSystemMetrics(SM_CXVSCROLL); // + (tR.Bottom - tR.Top) div 2;

      end;

        // Will all panels fit in at maximum size?
    if (iParts[iPanelCount - 1] > tR.Right) then
            // Draw all panels at min width
    else begin
            // Spring the spring panel to fit:
      if (iSpringIndex = -1) then
        iSpringIndex := iPanelCount - 1;

      iParts[iSpringIndex] := iParts[iSpringIndex] + (tR.Right - iParts[iPanelCount - 1]);

      for i := iSpringIndex + 1 to iPanelCount - 1 do
        with Items[i] do begin
          if Visible then
            if (FitToContent) then
              iParts[i] := iParts[i - 1] + MinWidth
            else
              iParts[i] := iParts[i - 1] + Width
          else
            iParts[i] := iParts[i - 1] + 0;

                    {if (ImageIndex > -1) and Visible then
                        //Add space for the icon:
                        iParts[i] := iParts[i] + m_iIconSize;   }

          if (i = iPanelCount - 1) and Visible then
            iParts[i] := iParts[i] + GetSystemMetrics(SM_CXVSCROLL); // + (tR.Bottom - tR.Top) div 2;

        end;
    end;

        {p:= Items[0];
        p.FSetWidth := iParts[0];

        For i := 1 To iPanelCount-1 do begin
            p:= Items[i];
            p.FSetWidth := iParts[i-1] - iParts[i - 2];
        end;    }

        // Set the sizes:
    for i := 0 to iPanelCount - 1 do
      with Items[i] do begin

        if (i = 0) then
          FPanelRect.Left := tR.Left
        else
          FPanelRect.Left := iParts[i - 1];

        if (i = iPanelCount - 1) then
          FPanelRect.Right := iParts[i]
        else
          FPanelRect.Right := iParts[i] - 2;

        FPanelRect.Top := tR.Top;
        FPanelRect.Bottom := tR.Bottom

      end;

  end;

end;

procedure TpbhStatusPanels.SetItem(Index: Integer; Value: TpbhStatusPanel);
begin
  inherited SetItem(Index, Value);
end;

procedure TpbhStatusPanels.Update(Item: TCollectionItem);
begin
  inherited;
  if Item <> nil then
    EvaluateIdealSize(Item);

  FStatusBar.Paint;

end;

{ TpbhStatusBar }

procedure TpbhStatusBar.Click;
var
  i: integer;
  tP: TPoint;
begin
  if SimplePanel then begin
    if Assigned(OnClick) then
      OnClick(Self)
  end else begin
    GetCursorPos(tP);
    tP := ScreenToClient(tP);
    for i := 0 to FPanels.Count - 1 do
      if PtInRect(FPanels[i].FPanelRect, tP) then begin
        if Assigned(FPanels[i].FOnClick) then
          FPanels[i].FOnClick(FPanels[i]);
        Exit;
      end;
  end;
end;

procedure TpbhStatusBar.CMColorChanged(var Msg: TMessage);
begin
  inherited;
  Invalidate;
end;

procedure TpbhStatusBar.CMFontChanged(var Msg: TMessage);
var
  R: TRect;
begin
  inherited;
  Canvas.Font.Assign(Font);
  DrawText(Canvas.Handle, PChar('Xy'), Length('Xy'), R, DT_CALCRECT);
  FMinHeight := R.Bottom - R.Top + 6;
  Invalidate;
end;

procedure TpbhStatusBar.CMHintShow(var Message: TCMHintShow);
var
  i: integer;
begin
  inherited;
  if SimplePanel then begin
    if Assigned(FOnHint) then
      FOnHint(Self);
    Message.HintInfo.HintStr := Hint;

  end else
    for i := 0 to FPanels.Count - 1 do
      if PtInRect(FPanels[i].FPanelRect, Message.HintInfo.CursorPos) then begin
        if Assigned(FPanels[i].FOnHint) then
          FPanels[i].FOnHint(FPanels[i]);
        Message.HintInfo.HintStr := FPanels[i].Hint;
        Message.HintInfo.CursorRect := FPanels[i].FPanelRect;
        Exit;
      end;
end;

constructor TpbhStatusBar.Create(AOwner: TComponent);
begin
  inherited;
  Color := clBtnFace;
  Height := m_iDefaultHeight;
  Align := alBottom;
  FPanels := TpbhStatusPanels.Create(Self);

  FSizeGrip := True;
  ParentFont := False;
  FUseSystemFont := True;
  InitSystemFont;

end;

procedure TpbhStatusBar.DblClick;
var
  i: integer;
  tP: TPoint;
begin
  if SimplePanel then begin
    if Assigned(OnDblClick) then
      OnDblClick(Self)
  end else begin
    GetCursorPos(tP);
    tP := ScreenToClient(tP);
    for i := 0 to FPanels.Count - 1 do
      if PtInRect(FPanels[i].FPanelRect, tP) then begin
        if Assigned(FPanels[i].FOnDblClick) then
          FPanels[i].FOnDblClick(FPanels[i]);
        Exit;
      end;
  end;
end;

destructor TpbhStatusBar.Destroy;
begin
  FPanels.Free;
  FSysFont.Free;
  inherited;
end;

procedure TpbhStatusBar.InitSystemFont;
var
  NonClientMetrics: TNonClientMetrics;
begin
  FSysFont := TFont.Create;
  NonClientMetrics.cbSize := SizeOf(NonClientMetrics);
  if SystemParametersInfo(SPI_GETNONCLIENTMETRICS, 0, @NonClientMetrics, 0) then
    FSysFont.Handle := CreateFontIndirect(NonClientMetrics.lfMenuFont);
end;

procedure TpbhStatusBar.WMNCHitTest(var Message: TWMNCHitTest);
var
  p: TPoint;
  tR: TRect;
begin
  inherited;
  if SizeGrip then
    if Message.Result = HTCLIENT then begin
      tR := GetClientRect;
      tR.Left := tR.Right - GetSystemMetrics(SM_CYHSCROLL);
      tR.Top := tR.Bottom - GetSystemMetrics(SM_CXVSCROLL);
      p := SmallPointToPoint(Message.Pos);
      p := ScreenToClient(p);
      if (p.x > tR.Left) and (p.y > tR.Top) then
        Message.Result := HTBOTTOMRIGHT ;
    end;

end;

procedure TpbhStatusBar.Paint;

  procedure DrawPanelBorder(c: TCanvas; p: TpbhStatusPanel; r: TRect);
  begin
    case p.Bevel of
      pbsSunkenBorder:
        with c do begin
          Pen.Color := p.Colors.BorderShadowColor;
          Polyline([Point(r.Left, r.Bottom), Point(r.Left, r.Top),
            Point(r.Right, r.Top)]);
          Pen.Color := p.Colors.BorderHighLightColor;
          Polyline([Point(r.Right - 1, r.Top), Point(r.Right - 1, r.Bottom - 1),
            Point(r.Left - 1, r.Bottom - 1)]);
        end;

      pbsFlatBorder:
        with c do begin
          Brush.Color := p.Colors.FlatBorderColor;
          FrameRect(r);
          Brush.Color := p.Colors.BackColor;
        end;
      pbsRaisedBorder:
        with c do begin
          Pen.Color := p.Colors.BorderHighLightColor;
          Polyline([Point(r.Left, r.Bottom), Point(r.Left, r.Top),
            Point(r.Right, r.Top)]);
          Pen.Color := p.Colors.BorderShadowColor;
          Polyline([Point(r.Right - 1, r.Top), Point(r.Right - 1, r.Bottom - 1),
            Point(r.Left - 1, r.Bottom - 1)]);
        end;

    end;
  end;

  procedure DrawPanelText(c: TCanvas; p: TpbhStatusPanel; r: TRect);
  begin
    case p.Alignment of
      taCenter: DrawText(c.Handle, PChar(p.Text), -1, r, DT_VCENTER or DT_SINGLELINE or DT_CENTER);
      taLeftJustify: DrawText(c.Handle, PChar(p.Text), -1, r, DT_VCENTER or DT_SINGLELINE);
      taRightJustify: DrawText(c.Handle, PChar(p.Text), -1, r, DT_VCENTER or DT_SINGLELINE or DT_RIGHT);
    end;
  end;

var
  i: integer;
  iX: integer;
  iY: integer;
  iSBW: integer;
  tR: TRect;
  tOR: TRect;
  tBR: TRect;
  tCR: TRect;
  bEnd: Boolean;
  p: TpbhStatusPanel;
  ClipRgn: HRGN;
  b: TBitmap;

begin
  if Height < FMinHeight then
    Height := FMinHeight;
  bEnd := False;
  tR := ClientRect;


  b := TBitmap.Create;
  b.Width := tR.Right;
  b.Height := tR.Bottom;

  b.Canvas.Brush.Color := Color;
  b.Canvas.FillRect(tR);

  CopyRect(tOR, tR);

  iSBW := GetSystemMetrics(SM_CXVSCROLL);

  FPanels.ResizePanels;

  if UseSystemFont then
    b.Canvas.Font.Assign(FSysFont)
  else
    b.Canvas.Font.Assign(Font);

  if SimplePanel then begin
    //if SizeGrip then
      //tR.Right := tR.Right - iSBW;

    DrawText(b.Canvas.Handle, PChar(SimpleText), Length(SimpleText), tR, DT_VCENTER or DT_SINGLELINE);

  end else
    for i := 0 to FPanels.Count - 1 do
    begin

      p := FPanels[i];

      tBR := p.FPanelRect;
      if (SimplePanel) then
        tBR.Right := tOR.Right
      else
        if (tBR.Right > tOR.Right) then begin
          tBR.Right := tOR.Right;
          bEnd := True;
        end;

      if p.Visible then begin
        if p.Style = pdsOwnerDraw then begin
          if not (csDesigning in ComponentState) then
            if Assigned(FOnDrawPanel) then
              FOnDrawPanel(Self, b.Canvas, p, p.FPanelRect);

        end else begin

          if not UseSystemFont then
            b.Canvas.Font.Assign(Font);

          DrawPanelBorder(b.Canvas, p, tBR);
          InflateRect(tBR, -1, -1);
          b.Canvas.Brush.Color := p.Colors.BackColor;
          b.Canvas.FillRect(tBR);

          InflateRect(tBR, -1, 0);
          if (i = Panels.Count - 1) and SizeGrip then
            tBR.Right := tBR.Right - iSBW;

          if (p.ImageIndex >= 0) and (Assigned(Images)) then begin

            ClipRgn := CreateRectRgnIndirect(tBR);
            SelectClipRgn(b.Canvas.Handle, ClipRgn);
            DeleteObject(ClipRgn);

                        // Calculate Top for icon:
            iY := tBR.Top + (tBR.Bottom - tBR.Top - m_iIconSize) div 2;


            case p.Alignment of
              taCenter:
                begin
                  DrawText(b.Canvas.Handle, PChar(p.Text), Length(p.Text), tCR, DT_CALCRECT);
                  iX := tCR.Right - tCR.Left + 20;
                  iX := tBR.Left + ((tBR.Right - tBR.Left) - iX) div 2;
                  Images.Draw(b.Canvas, iX, iY, p.ImageIndex);
                  tBR.Left := iX + m_iIconSize + 4;
                  DrawText(b.Canvas.Handle, PChar(p.Text), -1, tBR, DT_VCENTER or DT_SINGLELINE);

                end;

              taLeftJustify:
                begin
                  iX := tBR.Left;
                  Images.Draw(b.Canvas, iX, iY, p.ImageIndex);
                                    // Draw the text:
                  if (Length(p.Text) > 0) then begin
                    tBR.Left := tBR.Left + m_iIconSize + 4;
                    DrawPanelText(b.Canvas, p, tBR);
                  end;
                end;
              taRightJustify:
                begin
                  iX := tBR.Right - m_iIconSize;
                  Images.Draw(b.Canvas, iX, iY, p.ImageIndex);
                                    // Draw the text:
                  if (Length(p.Text) > 0) then begin
                    tBR.Right := iX - 2;
                    DrawPanelText(b.Canvas, p, tBR);
                  end;

                end;
            end;
            SelectClipRgn(b.Canvas.Handle, 0);

          end else

            DrawPanelText(b.Canvas, p, tBR);

        end;
      end;
      if bEnd then
        Break;

    end;

  if (SizeGrip) and (GetParentForm(Self).WindowState = wsNormal) then begin
        //maximized windows don't have sizegrips
    CopyRect(tOR, tR);
    tOR.Left := tOR.Right - iSBW;
    tOR.Top := tOR.Bottom - GetSystemMetrics(SM_CYHSCROLL);
    ClipRgn := CreateRectRgn(tOR.Left + 3, tOR.Top + 3, tOR.Right, tOR.Bottom);
    SelectClipRgn(b.Canvas.Handle, ClipRgn);
    DeleteObject(ClipRgn);

    b.Canvas.FillRect(tOR);
    OffsetRect(tOR, -1, -1);

    DrawFrameControl(b.Canvas.Handle, tOR, DFC_SCROLL, DFCS_SCROLLSIZEGRIP);
  end;

  Canvas.Draw(0, 0, b);
  b.Free;
end;

procedure TpbhStatusBar.SetImages(const Value: TImageList);
begin
  FImages := Value;
  Invalidate;
end;

procedure TpbhStatusBar.SetPanels(const Value: TpbhStatusPanels);
begin
  FPanels := Value;
  Invalidate;
end;

procedure TpbhStatusBar.SetSimplePanel(const Value: Boolean);
begin
  FSimplePanel := Value;
  Invalidate;
end;

procedure TpbhStatusBar.SetSimpleText(const Value: string);
begin
  FSimpleText := Value;
  if SimplePanel then
    Invalidate;
end;

procedure TpbhStatusBar.SetSizeGrip(const Value: Boolean);
begin
  FSizeGrip := Value;
  Invalidate;
end;

procedure TpbhStatusBar.SetUseSystemFont(const Value: Boolean);
begin

  FUseSystemFont := Value;
  if FUseSystemFont then
    FMinHeight := m_iDefaultHeight
  else
    Canvas.Font.Assign(Font);
  Invalidate;
end;

{ TpbhPanelBorderColor }

procedure TpbhPanelColors.Assign(Source: TPersistent);
var
  SourceColor: TpbhPanelColors absolute Source;
begin
  if Source is TpbhPanelColors then begin
    BackColor := SourceColor.BackColor;
    BorderHighLightColor := SourceColor.BorderHighLightColor;
    BorderShadowColor := SourceColor.BorderShadowColor;
    FlatBorderColor := SourceColor.FlatBorderColor;
  end else
    inherited;

end;

constructor TpbhPanelColors.Create(AOwner: TpbhStatusPanel);
begin
  inherited Create;
  FOwner := AOwner;
  FBackColor := clBtnFace;
  FBorderHighLightColor := clBtnHighLight;
  FBorderShadowColor := clBtnShadow;
  FFlatBorderColor := clBtnHighlight;
end;

procedure TpbhPanelColors.SetBackColor(const Value: TColor);
begin
  if FBackColor <> Value then begin
    FBackColor := Value;
    TpbhStatusPanel(FOwner).Changed(False);
  end;
end;

procedure TpbhPanelColors.SetBorderHighLightColor(const Value: TColor);
begin
  if FBorderHighLightColor <> Value then begin
    FBorderHighLightColor := Value;
    TpbhStatusPanel(FOwner).Changed(False);
  end;
end;

procedure TpbhPanelColors.SetBorderShadowColor(const Value: TColor);
begin
  if FBorderShadowColor <> Value then begin
    FBorderShadowColor := Value;
    TpbhStatusPanel(FOwner).Changed(False);
  end;
end;

procedure TpbhPanelColors.SetFlatBorderColor(const Value: TColor);
begin
  if FFlatBorderColor <> Value then begin
    FFlatBorderColor := Value;
    TpbhStatusPanel(FOwner).Changed(False);
  end;
end;



end.

